@@ -61,6 +61,11 @@ def getSqlaRelationshipFieldNames(cls):
 
 
 def addTupleType(cls):
+    if cls.tupleName() is None:
+        raise Exception(
+            "You must set __tupleType__ to a unique string for this tuple"
+        )
+
     if not cls.tupleName():
         cls.__tupleType__ = "%s.%s" % (cls.__module__, cls.__name__)
 
@@ -150,7 +155,8 @@ def addTupleType(cls):
 
     if hasSlots and hasFieldNames:
         raise Exception(
-            "Only one of __slots__ or __fieldNames__ can be defined" " but not both"
+            "Only one of __slots__ or __fieldNames__ can be defined"
+            " but not both"
         )
 
     # If field names already exist, then work off these
@@ -251,7 +257,9 @@ def removeTuplesForTupleNames(tupleNames):
     def filt(cls):
         return cls.tupleName() not in tupleNames
 
-    tupleShortNames = [cls.__tupleTypeShort__ for cls in TUPLE_TYPES if filt(cls)]
+    tupleShortNames = [
+        cls.__tupleTypeShort__ for cls in TUPLE_TYPES if filt(cls)
+    ]
 
     # Remove from tuple types
     TUPLE_TYPES = list(filter(filt, TUPLE_TYPES))
@@ -350,7 +358,8 @@ class IntTupleFieldValidator(TupleFieldValidatorABC):
 
         if self._high is not None and self._high < value:
             raise ValueError(
-                f"Field {fieldName}," f" Value {value} is greater than {self._high}"
+                f"Field {fieldName},"
+                f" Value {value} is greater than {self._high}"
             )
 
 
@@ -455,7 +464,9 @@ class _TupleToPlainJsonMixin:
     def smallJsonDictToTuple(jsonDict: dict) -> "Tuple":
         tupleShortType = jsonDict.get("_tt")
         if not tupleShortType:
-            raise Exception("Tuple.smallJsonDictToTuple: jsonDict has no _tt field")
+            raise Exception(
+                "Tuple.smallJsonDictToTuple: jsonDict has no _tt field"
+            )
 
         Tuple_ = TUPLE_TYPES_BY_SHORT_NAME.get(tupleShortType)
         if not Tuple_:
@@ -484,11 +495,7 @@ class _TupleToPlainJsonMixin:
             # elif isinstance(value, WKBElement):
             #     return convertFromWkbElement(value)
 
-            if (
-                isinstance(value, str)
-                and "\n" not in value
-                and re.match(value, ISO8601_REGEXP)
-            ):
+            if isinstance(value, str) and re.match(ISO8601_REGEXP, value):
                 return strptime(value, ISO8601)
 
             return value
@@ -606,11 +613,7 @@ class _TupleToPlainJsonMixin:
             if value in (None, ""):
                 return value
 
-            if (
-                isinstance(value, str)
-                and "\n" not in value
-                and re.match(value, ISO8601_REGEXP)
-            ):
+            if isinstance(value, str) and re.match(ISO8601_REGEXP, value):
                 return strptime(value, ISO8601)
 
             if isinstance(value, list):
@@ -671,19 +674,27 @@ class _TupleToPlainJsonMixin:
                                 f" {valueTypeHint}, got {type(value)} instead"
                             )
 
-                        data[key] = TupleClass.restfulJsonDictToTupleWithValidation(
+                        data[
+                            key
+                        ] = TupleClass.restfulJsonDictToTupleWithValidation(
                             item, valueTypeHint
                         )
 
                 else:
-                    keyTypeExtraHint, valueTypeExtraHint = get_args(extraTypeHint)
-                    keyTypeValidator = getFieldValidatorFromExtraHint(keyTypeExtraHint)
+                    keyTypeExtraHint, valueTypeExtraHint = get_args(
+                        extraTypeHint
+                    )
+                    keyTypeValidator = getFieldValidatorFromExtraHint(
+                        keyTypeExtraHint
+                    )
                     valueTypeValidator = getFieldValidatorFromExtraHint(
                         valueTypeExtraHint
                     )
 
                     for key, item in value.items():
-                        raiseIfNotType(fieldName, keyTypeHint, key, keyTypeValidator)
+                        raiseIfNotType(
+                            fieldName, keyTypeHint, key, keyTypeValidator
+                        )
                         raiseIfNotType(
                             fieldName, valueTypeHint, item, valueTypeValidator
                         )
@@ -699,7 +710,9 @@ class _TupleToPlainJsonMixin:
         newTuple = TupleClass()
 
         for fieldName in TupleClass.__fieldNames__:
-            setattr(newTuple, fieldName, convert(fieldName, jsonDict.get(fieldName)))
+            setattr(
+                newTuple, fieldName, convert(fieldName, jsonDict.get(fieldName))
+            )
 
         return newTuple
 
@@ -759,7 +772,9 @@ class Tuple(Jsonable, _TupleToPlainJsonMixin, _TupleToSqlaJsonMixin):
 
             elif isinstance(tupleField, InstrumentedAttribute):
                 default = (
-                    self.__table__.c[name].default if name in self.__table__.c else None
+                    self.__table__.c[name].default
+                    if name in self.__table__.c
+                    else None
                 )
 
                 assign = (
@@ -851,7 +866,9 @@ class Tuple(Jsonable, _TupleToPlainJsonMixin, _TupleToSqlaJsonMixin):
         )
 
         vals = ["type = %s," % self.tupleType()]
-        vals.extend(["%s = %s," % (name, val(name)) for name in self.__fieldNames__])
+        vals.extend(
+            ["%s = %s," % (name, val(name)) for name in self.__fieldNames__]
+        )
 
         return "\n".join(vals)
 
@@ -880,7 +897,8 @@ class TupleHash(object):
 
             elif isinstance(val, (list, set)):
                 newItems = [
-                    TupleHash(v)._key() if isinstance(v, Tuple) else v for v in val
+                    TupleHash(v)._key() if isinstance(v, Tuple) else v
+                    for v in val
                 ]
                 val = tuple(newItems)
 
